#include "Ticker10.h"
#include "arduino.h"
#include <PID_v1.h>
#include <WiFi.h>
#include <AsyncTCP.h>
#include <SPIFFS.h>
#include <ESPAsyncWebServer.h>

#define ADC 27 // вход с термодатчика
#define SINCHRO 12 // вход синхронизации 50 Гц
#define FON 13 // вход с датчика фона
#define OUT 33 // выход на симистор
#define LED_BLUE 2 // светодиод синий
#define LED_RED 15 // светодиод красный

Ticker timer_sym; // таймер управления симистором

const char* ssid     = "Second-solder";
const char* password = "";

// Глобальные переменные
_Bool MODE; // режим работы: 0 - параметрирование, 1 - регулирование
_Bool SLEEP; // разрешение ухода в сон - 1, запрет - 0
uint8_t CONTROL; // режим управления: 0 - PARAMETR, 1 - ZERO, 2 - DIMMER, 3 - PID
uint16_t set_termo;  // заданная температура, град
uint16_t gyst_termo; // гистерезис регулирования температуры, град С
int16_t corr_termo; // коррекция температуры в + или - градусов
_Bool sleep_key; // разрешение перехода в сон, 1 - разрешено, 0 - запрещено
uint16_t sleep_delay; // задержка включения режима сна, сек
uint16_t sleep_val; // насколько снижается температура в режиме сна
uint16_t r1_t1; // сопротивление термистора при градуировочной температуре T1
uint16_t t1_r1; // градуировочная температура T1
uint16_t r2_t2; // сопротивление термистора при градуировочной температуре T2
uint16_t t2_r2; // градуировочная температура T2
uint32_t Kp; // коэффицент КP ПИД*1000
uint32_t Ki; // коэффицент КI ПИД*1000
uint32_t Kd; // коэффицент КD ПИД*1000
uint32_t TR; // коэффициент пересчета кода напряжения на термодатчике в его сопротивление * 1000
uint16_t gain; // коэффициент усиления в режиме регулирования DIMMER * 1000
double Setpoint, Input, Output; // переменные для PID
uint16_t tl431 = 1439; // напряжение стабилизации tl431 1,24В 
uint16_t ang_s_max = 800; // максимально допустимый угол открытия симистора в 10 мксек отсчетах
uint16_t s_imp = 100; // длительность импульса открытия симистора в 10 мксек отсчетах

// рабочие переменные
int16_t ang_s; // угол открытия симистора в 10 мксек отсчетах
double set_termo_work; // рабочая заданная температура
double set_termo_sleep; // сниженная заданная температура
double meas_res; // измеренное сопротивление термистора
double meas_termo; // измеренная температура, град
double dif_termo; // разница между заданной и измеренной температурами, град
double k; // коэффициент k в уравнении y = kx + b
double b; // коэффициент b в уравнении y = kx + b
double Kp_real; // дробный коэффицент КP ПИД
double Ki_real; // дробный коэффицент КI ПИД
double Kd_real; // дробный коэффицент КD ПИД
double TR_real; // дробный коэффициент пересчета кода напряжения на термодатчике в его сопротивление 
double gain_real; // дробный коэффициент усиления в режиме регулирования DIMMER 
unsigned long timing; // переменная для хранения точки отсчета для режима сна
unsigned long mill_time; // для пересчета millis в секунды

void IRAM_ATTR ISR_StartT1(); // запуск таймера установки угла открытия симистора 
void ISR_StopT1(); // включение импульса открытия симистора 
void ISR_StartT2(); // отключение импульса открытия симистора   

       void IRAM_ATTR ISR_StartT1() {
        if (MODE != 0) { 
            digitalWrite(OUT, LOW);
            if (ang_s < ang_s_max) timer_sym.once_ms(ang_s, ISR_StopT1); // угол включения симистра 
        }
      }
      void ISR_StopT1() { // старт импульса открытия симистора 10 mS
        if (MODE != 0) {
            digitalWrite(OUT, HIGH);
            timer_sym.once_ms(s_imp, ISR_StartT2);
        }
      }
      void ISR_StartT2() {  // стоп импульса открытия симистора 
        if (MODE != 0) digitalWrite(OUT, LOW);    
      }
    
void SET_TERM(); // установка заданной температуры с учетом коррекции и состояния режима сна

      void SET_TERM() {
          if ((digitalRead(FON) != 0) && (sleep_key != 0)) {
          mill_time = (millis() - timing)/1000; 
          if (mill_time > sleep_delay){ 
          timing = millis(); 
          set_termo_work = (double)(set_termo_sleep + corr_termo); 
          }
        }    
        else set_termo_work = (double)(set_termo + corr_termo); 
        if (set_termo_work < 0) set_termo_work = 0;
      }
// Параметры ПИД
PID myPID(&Input, &Output, &Setpoint, Kp_real, Ki_real, Kd_real, REVERSE); // установка параметров ПИД

// Настройки режима параметрирования
      AsyncWebServer server(80);
    
String inputMessage = "0.0";
String lastSpeed;

const char* PARAM_INT1 = "inputInt1";
const char* PARAM_INT2 = "inputInt2";
const char* PARAM_INT3 = "inputInt3";
const char* PARAM_INT4 = "inputInt4";
const char* PARAM_INT5 = "inputInt5";
const char* PARAM_INT6 = "inputInt6";
const char* PARAM_INT7 = "inputInt7";
const char* PARAM_INT8 = "inputInt8";
const char* PARAM_INT9 = "inputInt9";
const char* PARAM_INT10 = "inputInt10";
const char* PARAM_INT11 = "inputInt11";
const char* PARAM_INT12 = "inputInt12";
const char* PARAM_INT13 = "inputInt13";
const char* PARAM_INT14 = "inputInt14";
const char* PARAM_INT15 = "inputInt15";
const char* PARAM_INT16 = "inputInt16";
const char index_html[] PROGMEM = R"rawliteral(

<!DOCTYPE HTML><html><head>
  <title>ESP Input Form</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    h2 {font-size: 3.0rem;}
    p {font-size: 3.0rem;}
    body {max-width: 600px; margin:0px auto; padding-bottom: 25px;}
    .switch {position: relative; display: inline-block; width: 120px; height: 68px} 
    .switch input {display: none}
    .slider {position: absolute; top: 0; left: 0; right: 0; bottom: 0; background-color: #ccc; border-radius: 6px}
    .slider:before {position: absolute; content: ""; height: 52px; width: 52px; left: 8px; bottom: 8px; background-color: #fff; -webkit-transition: .4s; transition: .4s; border-radius: 3px}
    input:checked+.slider {background-color: #b30000}
    input:checked+.slider:before {-webkit-transform: translateX(52px); -ms-transform: translateX(52px); transform: translateX(52px)}
  </style>
  <h1>%SPEED% Параметры </h1>
   
  <script>
    function submitMessage() {
      alert("Saved value to ESP SPIFFS");
      setTimeout(function(){ document.location.reload(false); }, 500);   
    }
  </script></head><body>
  <form action="/get" target="hidden-form">
    Заданная температура, &deg;  (установлено <font style="color: Red"><font size="5">%inputInt1%</font></font>): <input type="number" name="inputInt1">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Коррекция температуры, &deg; (установлено <font style="color: Red"><font size="5">%inputInt2%</font></font>): <input type="number" name="inputInt2">
    <input type="submit" value="Submit" onclick="submitMessage()">
    </form><br>
  <form action="/get" target="hidden-form">
    Гистерезис +/-,  &deg; (установлено <font style="color: Red"><font size="5">%inputInt3%</font></font>): <input type="number" name="inputInt3">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Режим регулирования, 1-ZERO, 2-DIMMER, 3-PID (установлено <font style="color: Red"><font size="5">%inputInt4%</font></font>): <input type="number" name="inputInt4">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Режим сна, 1-есть, 0-нет (установлено <font style="color: Red"><font size="5">%inputInt5%</font></font>): <input type="number" name="inputInt5">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Время ухода в сон,сек (установлено <font style="color: Red"><font size="5">%inputInt6%</font></font>): <input type="number" name="inputInt6">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Снижение температуры во сне, &deg; (установлено <font style="color: Red"><font size="5">%inputInt7%</font></font>): <input type="number" name="inputInt7">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Градуировочная температура 1, &deg; (установлено <font style="color: Red"><font size="5">%inputInt8%</font></font>): <input type="number" name="inputInt8">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Сопротивление при градуировочной температуре 1, &#8486; (установлено <font style="color: Red"><font size="5">%inputInt9%</font></font>): <input type="number" name="inputInt9">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Градуировочная температура 2, &deg; (установлено <font style="color: Red"><font size="5">%inputInt10%</font></font>): <input type="number" name="inputInt10">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Сопротивление при градуировочной температуре 2, &#8486; (установлено <font style="color: Red"><font size="5">%inputInt11%</font></font>): <input type="number" name="inputInt11">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Пропорциональный коэффициент ПИД / 1000 (установлено <font style="color: Red"><font size="5">%inputInt12%</font></font>): <input type="number" name="inputInt12">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Инетегральный коэффициент ПИД / 1000 (установлено <font style="color: Red"><font size="5">%inputInt13%</font></font>): <input type="number" name="inputInt13">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Дифференциальный коэффициент ПИД /1000 (установлено <font style="color: Red"><font size="5">%inputInt14%</font></font>): <input type="number" name="inputInt14">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Коэффициент пересчета кода напряжения в температуру /1000 (установлено <font style="color: Red"><font size="5">%inputInt15%</font></font>): <input type="number" name="inputInt15">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Коэффициент усиления в режиме регулирования DIMMER /1000 (установлено <font style="color: Red"><font size="5">%inputInt16%</font></font>): <input type="number" name="inputInt16">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <iframe style="display:none" name="hidden-form"></iframe>
</body></html>)rawliteral";

void notFound(AsyncWebServerRequest *request) {
  request->send(404, "text/plain", "Not found");
}

String readFile(fs::FS &fs, const char * path){
  //Serial.printf("Reading file: %s\r\n", path);
  File file = fs.open(path, "r");
  if(!file || file.isDirectory()){
    //Serial.println("- empty file or failed to open file");
    return String();
  }
  //Serial.println("- read from file:");
  String fileContent;
  while(file.available()){
    fileContent+=String((char)file.read());
  }
  file.close();
  //Serial.println(fileContent);
  return fileContent;
}

void writeFile(fs::FS &fs, const char * path, const char * message){
  //Serial.printf("Writing file: %s\r\n", path);
  File file = fs.open(path, "w");
  if(!file){
    //Serial.println("- failed to open file for writing");
    return;
  }
  if(file.print(message)){
    //Serial.println("- file written");
  } else {
    //Serial.println("- write failed");
  }
  file.close();
}

// Replaces placeholder with stored values
String processor(const String& var){
  if(var == "inputInt1"){
    return readFile(SPIFFS, "/inputInt1.txt");
  }
  if(var == "inputInt2"){
    return readFile(SPIFFS, "/inputInt2.txt");
  }
  if(var == "inputInt3"){
    return readFile(SPIFFS, "/inputInt3.txt");
  }
  if(var == "inputInt4"){
    return readFile(SPIFFS, "/inputInt4.txt");
  }
  if(var == "inputInt5"){
    return readFile(SPIFFS, "/inputInt5.txt");
  }
  if(var == "inputInt6"){
    return readFile(SPIFFS, "/inputInt6.txt");
  }
  if(var == "inputInt7"){
    return readFile(SPIFFS, "/inputInt7.txt");
  }
  if(var == "inputInt8"){
    return readFile(SPIFFS, "/inputInt8.txt");
  }
  if(var == "inputInt9"){
    return readFile(SPIFFS, "/inputInt9.txt");
  }
  if(var == "inputInt10"){
    return readFile(SPIFFS, "/inputInt10.txt");
  }
  if(var == "inputInt11"){
    return readFile(SPIFFS, "/inputInt11.txt");
  }
  if(var == "inputInt12"){
    return readFile(SPIFFS, "/inputInt12.txt");
  }
  if(var == "inputInt13"){
    return readFile(SPIFFS, "/inputInt13.txt");
  }
  if(var == "inputInt14"){
    return readFile(SPIFFS, "/inputInt14.txt");
  }
  if(var == "inputInt15"){
    return readFile(SPIFFS, "/inputInt15.txt");
  }
  if(var == "inputInt16"){
    return readFile(SPIFFS, "/inputInt16.txt");
  }
  if(var == "SPEED"){
  return lastSpeed;
  }
  return String();
}
   
void setup() {
  
    pinMode(SINCHRO, INPUT);
    pinMode(FON, INPUT);
    pinMode(OUT, OUTPUT_OPEN_DRAIN);
    pinMode(LED_BLUE, OUTPUT_OPEN_DRAIN);
    pinMode(LED_RED, OUTPUT_OPEN_DRAIN);
    digitalWrite(LED_RED, HIGH);
    digitalWrite(LED_BLUE, HIGH);
    digitalWrite(OUT, LOW);
    
    MODE = digitalRead(FON); // Выбор режима на старте
    if (MODE != 0) attachInterrupt(digitalPinToInterrupt(SINCHRO), ISR_StartT1, CHANGE); // синхронизация от сети
       
    if (MODE == 0) {
      CONTROL = 0; // если включен режим параметрирования, то в loop ничего не делается
      Serial.begin(9600);
      
       //Serial.print("Setting AP (Access Point)…");
      // Remove the password parameter, if you want the AP (Access Point) to be open
      WiFi.softAP(ssid, password);

      IPAddress IP = WiFi.softAPIP();
      Serial.print("AP IP address: ");
      Serial.println(IP);

     // Send web page with input fields to client
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", index_html, processor);
  });
    // Send a GET request to <ESP_IP>/get?inputString=<inputMessage>
  server.on("/get", HTTP_GET, [] (AsyncWebServerRequest *request) {
    String inputMessage;
    // GET inputInt1 value on <ESP_IP>/get?inputInt1=<inputMessage>
      if (request->hasParam(PARAM_INT1)) {
      inputMessage = request->getParam(PARAM_INT1)->value();
      writeFile(SPIFFS, "/inputInt1.txt", inputMessage.c_str());
    }
    // GET inputInt2 value on <ESP_IP>/get?inputInt2=<inputMessage>
    else if (request->hasParam(PARAM_INT2)) {
      inputMessage = request->getParam(PARAM_INT2)->value();
      writeFile(SPIFFS, "/inputInt2.txt", inputMessage.c_str());
    }
    // GET inputInt3 value on <ESP_IP>/get?inputInt3=<inputMessage>
    else if (request->hasParam(PARAM_INT3)) {
      inputMessage = request->getParam(PARAM_INT3)->value();
      writeFile(SPIFFS, "/inputInt3.txt", inputMessage.c_str());
    }
   // GET inputInt4 value on <ESP_IP>/get?inputInt4=<inputMessage>
      if (request->hasParam(PARAM_INT4)) {
      inputMessage = request->getParam(PARAM_INT4)->value();
      writeFile(SPIFFS, "/inputInt4.txt", inputMessage.c_str());
    }
    // GET inputInt5 value on <ESP_IP>/get?inputInt5=<inputMessage>
    else if (request->hasParam(PARAM_INT5)) {
      inputMessage = request->getParam(PARAM_INT5)->value();
      writeFile(SPIFFS, "/inputInt5.txt", inputMessage.c_str());
    }
    // GET inputInt6 value on <ESP_IP>/get?inputInt6=<inputMessage>
    else if (request->hasParam(PARAM_INT6)) {
      inputMessage = request->getParam(PARAM_INT6)->value();
      writeFile(SPIFFS, "/inputInt6.txt", inputMessage.c_str());
    }
    // GET inputInt7 value on <ESP_IP>/get?inputInt7=<inputMessage>
      if (request->hasParam(PARAM_INT7)) {
      inputMessage = request->getParam(PARAM_INT7)->value();
      writeFile(SPIFFS, "/inputInt7.txt", inputMessage.c_str());
    }
    // GET inputInt8 value on <ESP_IP>/get?inputInt8=<inputMessage>
    else if (request->hasParam(PARAM_INT8)) {
      inputMessage = request->getParam(PARAM_INT8)->value();
      writeFile(SPIFFS, "/inputInt8.txt", inputMessage.c_str());
    }
    // GET inputInt9 value on <ESP_IP>/get?inputInt9=<inputMessage>
    else if (request->hasParam(PARAM_INT9)) {
      inputMessage = request->getParam(PARAM_INT9)->value();
      writeFile(SPIFFS, "/inputInt9.txt", inputMessage.c_str());
    }
   // GET inputInt10 value on <ESP_IP>/get?inputInt10=<inputMessage>
      if (request->hasParam(PARAM_INT10)) {
      inputMessage = request->getParam(PARAM_INT10)->value();
      writeFile(SPIFFS, "/inputInt10.txt", inputMessage.c_str());
    }
    // GET inputInt11 value on <ESP_IP>/get?inputInt11=<inputMessage>
    else if (request->hasParam(PARAM_INT11)) {
      inputMessage = request->getParam(PARAM_INT11)->value();
      writeFile(SPIFFS, "/inputInt11.txt", inputMessage.c_str());
    }
    // GET inputInt12 value on <ESP_IP>/get?inputInt12=<inputMessage>
    else if (request->hasParam(PARAM_INT12)) {
      inputMessage = request->getParam(PARAM_INT12)->value();
      writeFile(SPIFFS, "/inputInt12.txt", inputMessage.c_str());
    }
    // GET inputInt13 value on <ESP_IP>/get?inputInt13=<inputMessage>
    else if (request->hasParam(PARAM_INT13)) {
      inputMessage = request->getParam(PARAM_INT13)->value();
      writeFile(SPIFFS, "/inputInt13.txt", inputMessage.c_str());
    }
    // GET inputInt14 value on <ESP_IP>/get?inputInt14=<inputMessage>
    else if (request->hasParam(PARAM_INT14)) {
      inputMessage = request->getParam(PARAM_INT14)->value();
      writeFile(SPIFFS, "/inputInt14.txt", inputMessage.c_str());
    }
    // GET inputInt15 value on <ESP_IP>/get?inputInt15=<inputMessage>
    else if (request->hasParam(PARAM_INT15)) {
      inputMessage = request->getParam(PARAM_INT15)->value();
      writeFile(SPIFFS, "/inputInt15.txt", inputMessage.c_str());
    }
    // GET inputInt16 value on <ESP_IP>/get?inputInt16=<inputMessage>
    else if (request->hasParam(PARAM_INT16)) {
      inputMessage = request->getParam(PARAM_INT16)->value();
      writeFile(SPIFFS, "/inputInt16.txt", inputMessage.c_str());
    }
    else {
      inputMessage = "No message sent";
    }
    //Serial.println(inputMessage);
    request->send(200, "text/text", inputMessage);
    });
  server.onNotFound(notFound);
  server.begin();
  }
 
    SPIFFS.begin(true);

// Читаем параметры
  set_termo = readFile(SPIFFS, "/inputInt1.txt").toInt(); // читаем заданную температуру
  corr_termo = readFile(SPIFFS, "/inputInt2.txt").toInt(); // читаем заданный корректор
  gyst_termo = readFile(SPIFFS, "/inputInt3.txt").toInt(); // читаем значение гистерезиса
  CONTROL = readFile(SPIFFS, "/inputInt4.txt").toInt(); // читаем выбор режима регулирования
  sleep_key = readFile(SPIFFS, "/inputInt5.txt").toInt(); // читаем выбор режима сна 
  sleep_delay = readFile(SPIFFS, "/inputInt6.txt").toInt(); // читаем значение задержки ухода в сон 
  sleep_val = readFile(SPIFFS, "/inputInt7.txt").toInt(); // читаем величину снижения температуры во сне
  t1_r1 = readFile(SPIFFS, "/inputInt8.txt").toInt(); // читаем градуировочную температуру 1
  r1_t1 = readFile(SPIFFS, "/inputInt9.txt").toInt(); // читаем сопротивление при градуировочной температуре 1
  t2_r2 = readFile(SPIFFS, "/inputInt10.txt").toInt(); // читаем градуировочную температуру 2
  r2_t2 = readFile(SPIFFS, "/inputInt11.txt").toInt(); // читаем сопротивление при градуировочной температуре 2
  Kp = readFile(SPIFFS, "/inputInt12.txt").toInt(); // читаем пропорциональный коэффицент ПИД
  Ki = readFile(SPIFFS, "/inputInt13.txt").toInt(); // читаем интегральный коэффицент ПИД
  Kd = readFile(SPIFFS, "/inputInt14.txt").toInt(); // читаем дифференциальный коэффицент ПИД
  TR = readFile(SPIFFS, "/inputInt15.txt").toInt(); // коэффициент пересчета кода напряжение на термодатчике в температуру жала*1000
  gain = readFile(SPIFFS, "/inputInt16.txt").toInt(); // коэффициент усиления в режиме регулирования DIMMER * 1000
  
    if (MODE != 0) {
      SPIFFS.end(); // в режиме регулирования файловую систему отключаем из-за конфликта прерываний 
      set_termo_sleep = set_termo - sleep_val; // расчет сниженной температуры для режима сна
      k = (double)(r2_t2 - r1_t1)/(double)(t2_r2 - t1_r1); // затем вычисляем коэффициенты зависимости сопротивления термистора от температуры, по двум точкам
      b = r2_t2 - (k * t2_r2); // по ним вычисляем то сопротивление, которое соответствует заданной температуре
      Kp_real = Kp/1000.0; // дробный КP для ПИД
      Ki_real = Ki/1000.0; // дробный КI для ПИД  
      Kd_real = Kd/1000.0; // дробный КD для ПИД
      TR_real = TR/1000.0; // дробный TR
      gain_real = gain/1000.0; //дробный gain
    
      digitalWrite(LED_BLUE, HIGH); // Если включен режим регулирования, то не горит 
    }
    else {
      CONTROL = 0; // В режиме параметрирования loop пустой.
      digitalWrite(LED_BLUE, LOW); // Если включен режим параметрирования, то горит
    }

    if ((MODE != 0) && (CONTROL == 3)) { 
      myPID.SetMode(AUTOMATIC); // запуск ПИД
      myPID.SetOutputLimits(0, ang_s_max); // установка минимума и максимума угла открытия симистора
      myPID.SetSampleTime(1); // установка времени выборки - 1 мсек  
    }
}

void loop() {
     
    switch(CONTROL) {
    case 0:
      break; // режим параметрирования
    case 1:
      SET_TERM();
      meas_res = (analogRead(ADC) - tl431)*TR_real; // пересчет кода напряжения на термодатчике в сопротивление  
      meas_termo = (meas_res - b)/k; // пересчет по графику сопротивление\температура
      if (set_termo_work > meas_termo) {
        ang_s = 0; // если измеренная температура ниже заданной, симистор включен
        digitalWrite(LED_RED, LOW); // и светодиод включен
      }
      else {
        ang_s = ang_s_max; // в противном случае симистор выключен
        digitalWrite(LED_RED, HIGH); // и светодиод выключен
      }
      break;
    case 2:
      SET_TERM();
      meas_res = (analogRead(ADC) - tl431)*TR_real; 
      meas_termo = (meas_res - b)/k; // пересчет по графику сопротивление\температура
      dif_termo = set_termo_work - meas_termo; 
      if (dif_termo > 0) digitalWrite(LED_RED, LOW); // светодиод включен
      else digitalWrite(LED_RED, HIGH); // светодиод выключен
      ang_s = int((set_termo_work - dif_termo) * gain_real); 
      if (ang_s < 0) ang_s = 0;
      break;
    case 3: 
      SET_TERM();
      Setpoint = set_termo_work;
      meas_res = (analogRead(ADC) - tl431)*TR_real; // пересчет кода напряжения на термодатчике в сопротивление
      Input = (meas_res - b)/k; // пересчет по графику сопротивление\температура
      dif_termo = set_termo_work - meas_termo; 
      if (dif_termo > 0) digitalWrite(LED_RED, LOW); // светодиод включен
      else digitalWrite(LED_RED, HIGH); // светодиод выключен
      myPID.Compute();
      ang_s = (int)Output;
      if (ang_s < 0) ang_s = 0;
      break; 
  } 
    
}
